#External:
import numpy as np
import math
import time
import h5netcdf.legacyapi as netCDF4_h5
import netCDF4
import h5py
import datetime
import copy
import os
from collections import OrderedDict

#Internal:
import indices_utils

def check_if_opens(dataset,default=False):
    if default: return False
    return True

def get_year_axis(dataset,default=False):
    if default: return np.array([]),np.array([])

    dimensions_list=dataset.dimensions.keys()
    time_dim=find_time_dim(dataset)
    date_axis = get_date_axis(dataset,time_dim)
    year_axis=np.array([date.year for date in date_axis])
    month_axis=np.array([date.month for date in date_axis])
    return year_axis, month_axis

def get_date_axis(dataset,time_dim,default=False):
    if default: return np.array([])
    
    units=dataset.variables[time_dim].units
    if 'calendar' in dataset.variables[time_dim].ncattrs():
        calendar=dataset.variables[time_dim].calendar
    else:
        calendar=None
    return get_date_axis_from_units_and_calendar(dataset.variables[time_dim][:],units,calendar)

def get_date_axis_from_units_and_calendar(time_axis,units,calendar,default=False):
    if default: return np.array([])

    if units=='day as %Y%m%d.%f':
        date_axis=get_date_axis_absolute(time_axis)
    else:
        date_axis=get_date_axis_relative(time_axis,units,calendar)
    return date_axis

def get_date_axis_relative(time_axis,units,calendar,default=False):
    if default: return np.array([])
    if calendar!=None:
        try:
            date_axis = netCDF4.num2date(time_axis,units=units,calendar=calendar)
        except ValueError:
            if (
                (units=='days since 0-01-01 00:00:00' and
                calendar=='365_day') or
                (units=='days since 0-1-1 00:00:00' and
                calendar=='365_day') 
                ):
                date_axis = netCDF4.num2date(time_axis-365.0,units='days since 1-01-01 00:00:00',calendar=calendar)
            else:
                raise
    else:
        date_axis = netCDF4.num2date(time_axis,units=units)
    return date_axis

def get_date_axis_absolute(time_axis,default=False):
    if default: return np.array([])
    return map(convert_to_date_absolute,time_axis)

def get_time(dataset,time_var='time',default=False):
    if default: return np.array([])
    time_dim=find_time_dim(dataset,time_var=time_var)
    time_axis, attributes=retrieve_dimension(dataset,time_dim)
    date_axis=create_date_axis_from_time_axis(time_axis,attributes)
    return date_axis

def get_time_axis_relative(date_axis,units,calendar,default=False):
    if default: return np.array([])
    if calendar!=None:
        try:
            time_axis = netCDF4.date2num(date_axis,units=units,calendar=calendar)
        except ValueError:
            if (
                (units=='days since 0-01-01 00:00:00' and
                calendar=='365_day') or
                (units=='days since 0-1-1 00:00:00' and
                calendar=='365_day') 
                ):
                time_axis = netCDF4.date2num(date_axis,units='days since 1-01-01 00:00:00',calendar=calendar)+365.0
            else:
                raise
    else:
        time_axis = netCDF4.date2num(date_axis,units=units)
    return time_axis

def convert_to_date_absolute(absolute_time):
    year=int(math.floor(absolute_time/1e4))
    remainder=absolute_time-year*1e4
    month=int(math.floor(remainder/1e2))
    remainder-=month*1e2
    day=int(math.floor(remainder))
    remainder-=day
    remainder*=24.0
    hour=int(math.floor(remainder))
    remainder-=hour
    remainder*=60.0
    minute=int(math.floor(remainder))
    remainder-=minute
    remainder*=60.0
    seconds=int(math.floor(remainder))
    return datetime.datetime(year,month,day,hour,minute,seconds)

def replicate_full_netcdf_recursive(dataset,output,transform=(lambda x,y,z:y),slices=dict(),
                                                check_empty=False,default=False):
    if default: return output

    for var_name in dataset.variables.keys():
        replicate_and_copy_variable(dataset,output,var_name,transform=transform,slices=slices,check_empty=check_empty)
    if len(dataset.groups.keys())>0:
        for group in dataset.groups.keys():
            output_grp=replicate_group(dataset,output,group)
            replicate_full_netcdf_recursive(dataset.groups[group],output_grp,transform=transform,slices=slices,check_empty=check_empty)
    return output

def dimension_compatibility(dataset,output,dim,default=False):
    if default: return False

    if (dim in output.dimensions.keys()
        and _dim_len(output,dim)!=_dim_len(dataset,dim)):
        #Dimensions mismatch, return without writing anything
        return False
    elif ( (dim in dataset.variables.keys() and
          dim in output.variables.keys()) and
          ( len(output.variables[dim])!=len(dataset.variables[dim]) or 
           (dataset.variables[dim][:]!=dataset.variables[dim][:]).any())):
        #Dimensions variables mismatch, return without writing anything
        return False
    else:
        return True

def check_dimensions_compatibility(dataset,output,var_name,exclude_unlimited=False,default=False):
    if default: return False
    for dim in dataset.variables[var_name].dimensions:
        #The dimensions might be in the parent group:
        if not dim in dataset.dimensions.keys():
            dataset_parent=dataset.parent
        elif not dim in dataset.variables.keys():
            #Important check for h5netcdf
            dataset_parent=dataset.parent
        else:
            dataset_parent=dataset

        if not dim in output.dimensions.keys():
            output_parent=output.parent
        else:
            output_parent=output

        if not _isunlimited(dataset_parent,dim) or not exclude_unlimited:
            if not dimension_compatibility(dataset_parent,output_parent,dim):
                return False
    return True

def _isunlimited(dataset,dim):
    if (isinstance(dataset,netCDF4_h5.Dataset) or
        isinstance(dataset,netCDF4_h5.Group)):
        var_list_with_dim=[var for var in dataset.variables.keys() 
                                if dim in dataset.variables[var].dimensions]
        if len(var_list_with_dim)==0:
            return False

        if np.all([dataset._h5group[var].maxshape[
                                    list(dataset.variables[var].dimensions).index(dim)]==None
                                for var in var_list_with_dim]):
            #If the maxshape of dimension for all variables with dimenion is None, it is unlimited!
            return True
        else:
            return False
    else:
        return dataset.dimensions[dim].isunlimited()

def _dim_len(dataset,dim):
    if (isinstance(dataset,netCDF4_h5.Dataset) or
        isinstance(dataset,netCDF4_h5.Group)):
        return dataset.dimensions[dim]
    else:
        return len(dataset.dimensions[dim])

def _datatype(dataset,var):
    if (isinstance(dataset,netCDF4_h5.Dataset) or
        isinstance(dataset,netCDF4_h5.Group)):
        dtype=dataset.variables[var].dtype
        if dtype=='object':
            return str
        else:
            return dtype
    else:
        return dataset.variables[var].datatype

def append_record(dataset,output,default=False):
    record_dimensions=dict()
    if default: return record_dimensions
    for dim in dataset.dimensions.keys():
        if (     dim in dataset.variables.keys()
             and dim in output.dimensions.keys()
             and dim in output.variables.keys()
             and _isunlimited(dataset,dim)):
             append_slice=slice(_dim_len(output,dim),_dim_len(output,dim)+
                                                          _dim_len(dataset,dim),1)
             ensure_compatible_time_units(output,dataset,dim)
             temp=dataset.variables[dim][:]
             output.variables[dim][append_slice]=temp
             record_dimensions[dim]={'append_slice':append_slice}
    return record_dimensions

def ensure_compatible_time_units(dataset,output,dim,default=False):
    if default: return None
    for time_desc in ['units','calendar']:
        if ( (     time_desc in output.variables[dim].ncattrs()
               and time_desc in dataset.variables[dim].ncattrs()) and
              (output.variables[dim].getncattr(time_desc)!=dataset.variables[dim].getncattr(time_desc))):
            raise 'time units and calendar must be the same when appending soft links'
    return 

def append_and_copy_variable(dataset,output,var_name,record_dimensions,datatype=None,fill_value=None,add_dim=None,chunksize=None,zlib=False,check_empty=False,default=False):
    if default: return output

    if len(set(record_dimensions.keys()).intersection(dataset.variables[var_name].dimensions))==0:
        #Variable does not contain a record dimension, return
        return output
   
    variable_size=min(dataset.variables[var_name].shape)
    storage_size=variable_size
    if '_h5ds' in dir(dataset):
        #Use the hdf5 library to find the real size of the stored array:
        variable_size=dataset.variables[var_name]._h5ds.size
        storage_size=dataset.variables[var_name]._h5ds.id.get_storage_size()

    if variable_size>0 and storage_size>0:
        max_request=450.0 #maximum request in Mb
        max_first_dim_steps=max(
                        int(np.floor(max_request*1024*1024/(32*np.prod(dataset.variables[var_name].shape[1:])))),
                        1)

        num_first_dim_chunk=int(np.ceil(dataset.variables[var_name].shape[0]/float(max_first_dim_steps)))
        for first_dim_chunk in range(num_first_dim_chunk):
            first_dim_slice=slice(first_dim_chunk*max_first_dim_steps,
                             min((first_dim_chunk+1)*max_first_dim_steps,dataset.variables[var_name].shape[0])
                             ,1)
            output=append_dataset_first_dim_slice(dataset,output,var_name,first_dim_slice,record_dimensions,check_empty)
    return output

def append_dataset_first_dim_slice(dataset,output,var_name,first_dim_slice,record_dimensions,check_empty):
    #Create a setitem tuple
    setitem_list=[ slice(0,_dim_len(dataset,dim),1) if not dim in record_dimensions.keys()
                                                               else record_dimensions[dim]['append_slice']
                                                              for dim in dataset.variables[var_name].dimensions]
    #Pick a first_dim_slice along the first dimension:
    setitem_list[0]=indices_utils.slice_a_slice(setitem_list[0],first_dim_slice)
    temp=dataset.variables[var_name][first_dim_slice,...]
    #Assign only if not masked everywhere:
    if not 'mask' in dir(temp) or not check_empty:
        output.variables[var_name].__setitem__(tuple(setitem_list),temp)
    else: 
        #Only write the variable if it is not empty:
        if not temp.mask.all():
            output.variables[var_name].__setitem__(tuple(setitem_list),temp)
    return output

def replicate_and_copy_variable(dataset,output,var_name,
                                datatype=None,fill_value=None,
                                add_dim=None,
                                chunksize=None,zlib=False,
                                transform=(lambda x,y,z:y),
                                slices=dict(),
                                check_empty=False,default=False):

    if default: return output

    if not isinstance(slices,dict):
        #assume it is a function that takes the dataset as input and outputs
        #a slicing dict
        comp_slices=slices(dataset)
    else:
        comp_slices=slices

    replicate_netcdf_var(dataset,output,var_name,
                        datatype=datatype,fill_value=fill_value,
                        add_dim=add_dim,
                        slices=comp_slices,
                        chunksize=chunksize,zlib=zlib)

    #Apply a transformation if dimensions are in slices:
    if set(comp_slices.keys()).issubset(dataset.variables[var_name].dimensions):
        transform(dataset,output,comp_slices)

    if len(dataset.variables[var_name].dimensions)==0:
        #scalar variable:
        value=dataset.variables[var_name][...]
        if not np.ma.is_masked(value):
            #if not masked, assign. Otherwise, do nothing
            output.variables[var_name][...]=value
        return output

    variable_size=min(dataset.variables[var_name].shape)
    storage_size=variable_size
    if '_h5ds' in dir(dataset):
        #Use the hdf5 library to find the real size of the stored array:
        variable_size=dataset.variables[var_name]._h5ds.size
        storage_size=dataset.variables[var_name]._h5ds.id.get_storage_size()

    if variable_size>0 and storage_size>0:
        max_request=450.0 #maximum request in Mb
        #max_request=9000.0 #maximum request in Mb

        #Create the output variable shape, allowing slices:
        var_shape=tuple([dataset.variables[var_name].shape[dim_id] if not dim in comp_slices.keys()
                                               else len(np.arange(dataset.variables[var_name].shape[dim_id])[comp_slices[dim]])
                                               for dim_id,dim in enumerate(dataset.variables[var_name].dimensions)])
        max_first_dim_steps=max(
                        int(np.floor(max_request*1024*1024/(32*np.prod(var_shape[1:])))),
                        1)

        num_first_dim_chunk=int(np.ceil(var_shape[0]/float(max_first_dim_steps)))

        for first_dim_chunk in range(num_first_dim_chunk):
            first_dim_slice=slice(first_dim_chunk*max_first_dim_steps,
                             min((first_dim_chunk+1)*max_first_dim_steps,var_shape[0])
                             ,1)
            output=copy_dataset_first_dim_slice(dataset,output,var_name,first_dim_slice,check_empty,slices=comp_slices)
    return output

def copy_dataset_first_dim_slice(dataset,output,var_name,first_dim_slice,check_empty,slices=dict()):
    combined_slices=slices.copy()
    first_dim=dataset.variables[var_name].dimensions[0]
    if first_dim in combined_slices:
        combined_slices[first_dim]=indices_utils.slice_a_slice(combined_slices[first_dim],first_dim_slice)
    else:
        combined_slices[first_dim]=first_dim_slice
                
    getitem_tuple=tuple([combined_slices[var_dim] if var_dim in combined_slices.keys()
                                                else slice(None,None,1) for var_dim in
                                                dataset.variables[var_name].dimensions])

    temp=dataset.variables[var_name][getitem_tuple]
    #Assign only if not masked everywhere:
    if not 'mask' in dir(temp) or not check_empty:
        output.variables[var_name][first_dim_slice,...]=temp
    else: 
        #Only write the variable if it is not empty:
        if not temp.mask.all():
            output.variables[var_name][first_dim_slice,...]=temp
    return output

def replicate_group(dataset,output,group_name,default=False):
    if default: return output
    output_grp=create_group(dataset,output,group_name)
    replicate_netcdf_file(dataset.groups[group_name],output_grp)
    return output_grp

def create_group(dataset,output,group_name,default=False):
    if default: return output
    if not group_name in output.groups.keys():
        output_grp=output.createGroup(group_name)
    else:
        output_grp=output.groups[group_name]
    return output_grp
    
def replicate_netcdf_file(dataset,output,default=False):
    if default: return output

    for att in dataset.ncattrs():
        att_val=dataset.getncattr(att)
        
        #This fix is for compatitbility with h5netcdf:
        if ( 'dtype' in dir(att_val) and
            att_val.dtype==np.dtype('O')):
            if len(att_val)==1:
                att_val=att_val[0]
            else:
                att_val=np.asarray(att_val,dtype='str')

        if 'encode' in dir(att_val):
            att_val=str(att_val.encode('ascii','replace'))
            
        if (not att in output.ncattrs() and
            att != 'cdb_query_temp'):
            try:
                setattr(output,att,att_val)
            except:
                output.setncattr(att,att_val)
    return output


def _is_dimension_present(dataset,dim):
    if dim in dataset.dimensions:
        return True
    elif dataset.parent is not None:
        return _is_dimension_present(dataset.parent,dim)
    else:
        return False

def replicate_netcdf_var_dimensions(dataset,output,var,
                        slices=dict(),
                        datatype=None,fill_value=None,add_dim=None,chunksize=None,zlib=False,default=False):
    if default: return output
    for dims in dataset.variables[var].dimensions:
        if not _is_dimension_present(output,dims) and _is_dimension_present(dataset,dims):
            if _isunlimited(dataset,dims):
                output.createDimension(dims,None)
            elif dims in slices.keys():
                output.createDimension(dims,len(np.arange(_dim_len(dataset,dims))[slices[dims]]))
            else:
                output.createDimension(dims,_dim_len(dataset,dims))
            if dims in dataset.variables:
                replicate_netcdf_var(dataset,output,dims,zlib=True,slices=slices)
                if dims in slices.keys():
                    output.variables[dims][:]=dataset.variables[dims][slices[dims]]
                else:
                    output.variables[dims][:]=dataset.variables[dims][:]
                if ('bounds' in output.variables[dims].ncattrs() and
                    output.variables[dims].getncattr('bounds') in dataset.variables.keys()):
                    var_bounds=output.variables[dims].getncattr('bounds')
                    if not var_bounds in output.variables.keys():
                        output=replicate_netcdf_var(dataset,output,var_bounds,zlib=True,slices=slices)
                        if dims in slices.keys():
                            getitem_tuple=tuple([slices[var_bounds_dim] if var_bounds_dim in slices.keys()
                                                                        else slice(None,None,1) for var_bounds_dim in
                                                                        dataset.variables[var_bounds].dimensions])
                            output.variables[var_bounds][:]=dataset.variables[var_bounds][getitem_tuple]
                        else:
                            output.variables[var_bounds][:]=dataset.variables[var_bounds][:]
            else:
                #Create a dummy dimension variable:
                dim_var = output.createVariable(dims,np.float,(dims,),chunksizes=(1,))
                if dims in slices.keys():
                    dim_var[:]=np.arange(_dim_len(dataset,dims))[slices[dims]]
                else:
                    dim_var[:]=np.arange(_dim_len(dataset,dims))
    return output

def replicate_netcdf_other_var(dataset,output,var,time_dim,default=False):
    if default: return output
    #Replicates all variables except specified variable:
    variables_list=[ other_var for other_var in variables_list_with_time_dim(dataset,time_dim)
                                if other_var!=var]
    for other_var in variables_list:
        output=replicate_netcdf_var(dataset,output,other_var)
    return output

def replicate_netcdf_var(dataset,output,var,
                        slices=dict(),
                        datatype=None,fill_value=None,add_dim=None,chunksize=None,zlib=False,default=False):
    if default: 
        #Create empty variable:
        output.createVariable(var,'d',())
        return output

    if not var in dataset.variables.keys():
        return output

    output=replicate_netcdf_var_dimensions(dataset,output,var,slices=slices)
    if var in output.variables.keys():
        #var is a dimension variable and does not need to be created:
        return output

    if datatype==None: datatype=_datatype(dataset,var)
    if (isinstance(datatype,netCDF4.CompoundType) and
        not datatype.name in output.cmptypes.keys()):
        datatype=output.createCompoundType(datatype.dtype,datatype.name)

    #Weird fix for strings:
    #if 'str' in dir(datatype) and 'S1' in datatype.str:
    #    datatype='S1'

    kwargs=dict()
    if (fill_value==None and 
        '_FillValue' in dataset.variables[var].ncattrs() and 
        datatype==_datatype(dataset,var)):
            kwargs['fill_value']=dataset.variables[var].getncattr('_FillValue')
    else:
        kwargs['fill_value']=fill_value

    if not zlib:
        if dataset.variables[var].filters()==None:
            kwargs['zlib']=False
        else:
            for item in dataset.variables[var].filters():
                kwargs[item]=dataset.variables[var].filters()[item]
    else:
        kwargs['zlib']=zlib
    
    if var not in output.variables.keys():
        dimensions=dataset.variables[var].dimensions
        time_dim=find_time_dim(dataset)
        if add_dim:
            dimensions+=(add_dim,)
        var_shape=tuple([dataset.variables[var].shape[dim_id] if not dim in slices.keys()
                                               else len(np.arange(dataset.variables[var].shape[dim_id])[slices[dim]])
                                               for dim_id,dim in enumerate(dimensions)])
        if chunksize==-1:
            chunksizes=tuple([1 if dim==time_dim else var_shape[dim_id] for dim_id,dim in enumerate(dimensions)])
        elif dataset.variables[var].chunking()=='contiguous':
            if kwargs['zlib']:
                chunksizes=tuple([1 if dim==time_dim else var_shape[dim_id] for dim_id,dim in enumerate(dimensions)])
            else:
                chunksizes=tuple([1 for dim_id,dim in enumerate(dimensions)])
        else:
            if len(set(dimensions).intersection(slices.keys()))>0:
                if kwargs['zlib']:
                    chunksizes=tuple([1 if dim==time_dim else var_shape[dim_id] for dim_id,dim in enumerate(dimensions)])
                else:
                    chunksizes=tuple([1 for dim_id,dim in enumerate(dimensions)])
            else:
                chunksizes=dataset.variables[var].chunking()
        kwargs['chunksizes']=chunksizes
        out_var=output.createVariable(var,datatype,dimensions,**kwargs)
    output = replicate_netcdf_var_att(dataset,output,var)
    return output
    #return out_var

def replicate_netcdf_var_att(dataset,output,var,default=False):
    if default: return output
    for att in dataset.variables[var].ncattrs():
        att_val=dataset.variables[var].getncattr(att)
        if isinstance(att_val,dict):
            atts_pairs=[(att+'.'+key,att_val[key]) for key in att_val.keys()]
        else:
            atts_pairs=[(att,att_val)]
        for att_pair in atts_pairs:
            if att_pair[0][0]!='_':
                if 'encode' in dir(att_pair[1]):
                    att_val=att_pair[1].encode('ascii','replace')
                else:
                    att_val=att_pair[1]
                if 'encode' in dir(att_pair[0]):
                    att=att_pair[0].encode('ascii','replace')
                else:
                    att=att_pair[0]
                try:
                    setattr(output.variables[var],att,att_val)
                except:
                    output.variables[var].setncattr(att,att_val)
    return output

def create_time_axis(dataset,output,time_axis,time_var='time',default=False):
    if default: return output
    #output.createDimension(time_dim,len(time_axis))
    time_dim=find_time_dim(dataset,time_var=time_var)
    output.createDimension(time_dim,None)
    time = output.createVariable(time_dim,'d',(time_dim,),chunksizes=(1,))
    if dataset==None:
        time.calendar='standard'
        time.units='days since '+str(time_axis[0])
    else:
        time.calendar=netcdf_calendar(dataset,time_var=time_var)
        time_var=find_time_var(dataset,time_var=time_var)
        time.units=str(dataset.variables[time_var].getncattr('units'))
    time[:]=time_axis
    return output

def create_time_axis_date(output,time_axis,units,calendar,time_dim='time'):
    output.createDimension(time_dim,None)
    time = output.createVariable(time_dim,'d',(time_dim,),chunksizes=(1,))
    time.calendar=calendar
    time.units=units
    time[:]=get_time_axis_relative(time_axis,time.units,time.calendar)
    return

def netcdf_calendar(dataset,time_var='time',default=False):
    calendar='standard'
    if default: return calendar

    time_var=find_time_var(dataset,time_var=time_var)
    if time_var!=None:
        if 'calendar' in dataset.variables[time_var].ncattrs():
            calendar=dataset.variables[time_var].getncattr('calendar')
        if 'encode' in dir(calendar):
            calendar=calendar.encode('ascii','replace')
    return calendar
    
def find_time_var(dataset,time_var='time',default=False):
    if default: return time_var
    var_list=dataset.variables.keys()
    return find_time_name_from_list(var_list,time_var)

def find_time_dim(dataset,time_var='time',default=False):
    if default: return time_var
    dim_list=dataset.dimensions.keys()
    return find_time_name_from_list(dim_list,time_var)

def find_time_name_from_list(list_of_names,time_var):
    try:
        return list_of_names[next(i for i,v in enumerate(list_of_names) if v.lower() == time_var)]
    except StopIteration:
        return None

def variables_list_with_time_dim(dataset,time_dim,default=False):
    if default: return []
    return [ var for var in dataset.variables.keys() if time_dim in dataset.variables[var].dimensions]

def find_dimension_type(dataset,time_var='time',default=False):
    dimension_type=OrderedDict()
    if default: return dimension_type

    time_dim=find_time_name_from_list(dataset.dimensions.keys(),time_var)
    for dim in dataset.dimensions.keys():
        if dim!=time_dim:
            dimension_type[dim]=_dim_len(dataset,dim)
    return dimension_type

def netcdf_time_units(dataset,time_var='time',default=False):
    units=None
    if default: return units
    time_var=find_time_var(dataset,time_var=time_var)
    if 'units' in dataset.variables[time_var].ncattrs():
        units=dataset.variables[time_var].getncattr('units')
    return units

def retrieve_dimension(dataset,dimension,default=False):
    attributes=dict()
    dimension_dataset=np.array([])
    if default: return dimension_dataset, attributes

    if dimension in dataset.variables.keys():
        #Retrieve attributes:
        for att in dataset.variables[dimension].ncattrs():
            attributes[att]=dataset.variables[dimension].getncattr(att)
        #If dimension is available, retrieve
        dimension_dataset = dataset.variables[dimension][...]
    else:
        #If dimension is not avaiable, create a simple indexing dimension
        dimension_dataset = np.arange(_dim_len(dataset,dimension))
    return dimension_dataset, attributes

def retrieve_dimension_list(dataset,var,default=False):
    dimensions=tuple()
    if default: return dimensions
    return dataset.variables[var].dimensions

def retrieve_dimensions_no_time(dataset,var,time_var='time',default=False):
    dimensions_data=dict()
    attributes=dict()
    if default: return dimensions_data,attributes
    dimensions=retrieve_dimension_list(dataset,var)
    time_dim=find_time_name_from_list(dimensions,time_var)
    for dim in dimensions:
        if dim != time_dim:
            dimensions_data[dim], attributes[dim]=retrieve_dimension(dataset,dim)
    return dimensions_data, attributes

def retrieve_variables(dataset,output,zlib=True,default=False):
    if default: return output
    for var_name in dataset.variables.keys():
        output=replicate_and_copy_variable(dataset,output,var_name,zlib=zlib,check_empty=False)
    return output

def retrieve_variables_no_time(dataset,output,time_dim,zlib=False,default=False):
    if default: return output
    for var in dataset.variables.keys():
        if ( (not time_dim in dataset.variables[var].dimensions) and 
             (not var in output.variables.keys())):
            replicate_and_copy_variable(dataset,output,var,zlib=zlib)
    return output

def find_time_dim_and_replicate_netcdf_file(dataset,output,time_var='time',default=False):
    if default: return find_time_dim(dataset,time_var=time_var,default=True), replicate_netcdf_file(dataset,output,default=True)
    return find_time_dim(dataset,time_var=time_var), replicate_netcdf_file(dataset,output)

def create_date_axis_from_time_axis(time_axis,attributes_dict,default=False):
    if default: return np.array([])

    calendar='standard'
    units=attributes_dict['units']
    if 'calendar' in attributes_dict.keys(): 
        calendar=attributes_dict['calendar']

    if units=='day as %Y%m%d.%f':
        date_axis=np.array(map(convert_to_date_absolute,native_time_axis))
    else:
        try:
            #Put cmip5_rewrite_time_axis here:
            date_axis=get_date_axis_relative(time_axis,units,calendar)
            #date_axis=netCDF4.num2date(time_axis,units=units,calendar=calendar)
        except TypeError:
            date_axis=np.array([]) 
    return date_axis

def retrieve_container(dataset,var,dimensions,unsort_dimensions,sort_table,max_request,time_var='time',file_name='',default=False):
    if default: return np.array([])
    remote_dimensions, attributes = retrieve_dimensions_no_time(dataset, var, time_var=time_var)

    indices=copy.copy(dimensions)
    unsort_indices=copy.copy(unsort_dimensions)
    for dim in remote_dimensions.keys():
        indices[dim], unsort_indices[dim] = indices_utils.prepare_indices(
                                            indices_utils.get_indices_from_dim(remote_dimensions[dim],indices[dim]))
    return grab_indices(dataset,var,indices,unsort_indices,max_request,file_name=file_name)

def grab_indices(dataset,var,indices,unsort_indices,max_request,file_name='',default=False):
    if default: return np.array([])
    dimensions=retrieve_dimension_list(dataset,var)
    return indices_utils.retrieve_slice(dataset.variables[var],indices,unsort_indices,dimensions[0],dimensions[1:],0,max_request)



import requests
import pydap.lib
from bs4 import BeautifulSoup

username='flalib'
openid='https://pcmdi.llnl.gov/esgf-idp/openid/flalib'
url='http://esgf-data1.ceda.ac.uk/thredds/dodsC/esg_dataroot/cmip5/output1/CCCma/CanAM4/amip/fx/atmos/fx/r0i0p0/v1/orog/orog_fx_CanAM4_amip_r0i0p0.nc'

session=requests.Session()
resp=session.get(url,allow_redirects=True)

soup=BeautifulSoup(resp.text,'lxml')
form = soup.find('form')
auth_url=form.get('action')
if auth_url[0]=='/':
    auth_url='/'.join(url.split('/')[:4])+auth_url
resp=session.get(auth_url,allow_redirects=True)

check_url=resp.url.split('?')[0]
print(check_url+'?openid='+openid)
headers = {
    "User-Agent":
            "Mozilla/5.0 (Windows NT 6.3; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/41.0.2272.101 Safari/537.36",
    "openid":openid
            }
resp=session.post(check_url,allow_redirects=True,headers=headers)
print(check_url+'?openid='+openid)
#url='https://pcmdi.llnl.gov/esgf-idp/idp/login.htm'
#resp=session.get(url,allow_redirects=True,headers=headers)
#print(resp.text)
#url='https://esgf-data1.ceda.ac.uk/esg-orp/j_spring_openid_security_check.htm?openid_identifier=https%3A%2F%2Fpcmdi.llnl.gov%2Fesgf-idp%2Fopenid%2Fflalib'
url='https://pcmdi.llnl.gov/esgf-idp/idp/openidServer.htm?openid.ns=http%3A%2F%2Fspecs.openid.net%2Fauth%2F2.0&openid.claimed_id=https%3A%2F%2Fpcmdi.llnl.gov%2Fesgf-idp%2Fopenid%2Fflalib&openid.identity=https%3A%2F%2Fpcmdi.llnl.gov%2Fesgf-idp%2Fopenid%2Fflalib&openid.return_to=https%3A%2F%2Fesgf-data1.ceda.ac.uk%2Fesg-orp%2Fj_spring_openid_security_check.htm&openid.realm=https%3A%2F%2Fesgf-data1.ceda.ac.uk%2F&openid.assoc_handle=1469481824113-399&openid.mode=checkid_setup&openid.ns.ext1=http%3A%2F%2Fopenid.net%2Fsrv%2Fax%2F1.0&openid.ext1.mode=fetch_request&openid.ext1.type.email=http%3A%2F%2Fopenid.net%2Fschema%2Fcontact%2Finternet%2Femail&openid.ext1.type.firstname=http%3A%2F%2Fopenid.net%2Fschema%2FnamePerson%2Ffirst&openid.ext1.type.lastname=http%3A%2F%2Fopenid.net%2Fschema%2FnamePerson%2Flast&openid.ext1.required=email%2Cfirstname%2Clastname'
resp=session.get(url,allow_redirects=True,headers=headers)
#print(resp.text)

soup=BeautifulSoup(resp.text,'lxml')
form = soup.find('form')
auth_url=form.get('action')
if auth_url[0]=='/':
    auth_url='/'.join(resp.url.split('/')[:4])+auth_url
print(auth_url)
payload={'password':'Salut1001!'}
resp=session.post(auth_url,data=payload,allow_redirects=True,headers=dict(Referer=url))
print(resp.text)

url='http://esgf-data1.ceda.ac.uk/thredds/dodsC/esg_dataroot/cmip5/output1/CCCma/CanAM4/amip/fx/atmos/fx/r0i0p0/v1/orog/orog_fx_CanAM4_amip_r0i0p0.nc'
resp=session.get(url,allow_redirects=True,headers=headers)
print(resp.status_code)
quit()

csrftoken = session.cookies['csrftoken']

payload={'openid_identifier':openid+'flalib', 'csrfmiddlewaretoken':csrftoken}
#payload={}
#for input in inputs:
#    name=input.get('name')
#    value=input.get('value')
#    if name=='openid_identifier':
#        payload.update({name:openid})
#    elif input.get('type') != 'submit':
#        payload.update({name:value})
print(payload)
resp=session.post(url,data=payload,allow_redirects=True,headers=dict(Referer=url))

soup=BeautifulSoup(resp.text,'lxml')
form = soup.find('form')
auth_url=form.get('action')
inputs = form.find_all('input')
payload={}
for input in inputs:
    name=input.get('name')
    value=input.get('value')
    payload.update({name:value})
print(payload)

resp=session.post(auth_url,data=payload,allow_redirects=True,headers=dict(Referer=url))
soup=BeautifulSoup(resp.text,'lxml')
form=soup.find('form')
print(auth_url)
submit_url=form.get('action')
if submit_url[0]=='/':
    submit_url='/'.join(auth_url.split('/')[:4])+submit_url
print(submit_url)

inputs = form.find_all('input')
payload={'password':'Salut1001!'}

resp=session.post(submit_url,data=payload,allow_redirects=True,headers=dict(Referer=url))
print(resp.text)
quit()
headers = {
    'user-agent': pydap.lib.USER_AGENT,
        'connection': 'close'}
url='http://esgf-data1.ceda.ac.uk/thredds/dodsC/esg_dataroot/cmip5/output1/CCCma/CanAM4/amip/fx/atmos/fx/r0i0p0/v1/orog/orog_fx_CanAM4_amip_r0i0p0.nc'
resp=session.get(url,allow_redirects=True,headers=headers)
print(resp.status_code)



import h5py
file_name='/data/home/laliberte/CDB/2.0/sea_ice_landfast/obs/ORA/out_CA_obs/Mercator/GLORYS2V3/reanalysis/day/seaIce/day/r1i1p1/v20160728/sic/sic_day_GLORYS2V3_reanalysis_199401-201412.nc'
test=h5py.File(file_name)
getitem_tuple=(slice(1023, 1075, 1), slice(0, 144, 1), slice(0, 223, 1))
data=test['sic'][getitem_tuple]
print(data.shape)



import requests
#import netcdf4_soft_links.remote_netcdf as remote_netcdf
import netcdf4_soft_links.esgf_pydap as esgf_pydap
cred={'username': None, 'openid': 'https://esgf-data.dkrz.de/esgf-idp/openid/flaliberte', 'password': 'Salut1001!', 'use_certificates': False}
url=[u'http://cordexesg.dmi.dk/thredds/dodsC/cordex_general/cordex/output/EUR-11/DMI/ICHEC-EC-EARTH/historical/r3i1p1/DMI-HIRHAM5/v1/day/pr/v20131119/pr_EUR-11_ICHEC-EC-EARTH_historical_r3i1p1_DMI-HIRHAM5_v1_day_19960101-20001231.nc', u'OPENDAP']

session=requests.Session()
#import sys
#sys.setrecursionlimit(100)
import matplotlib.pyplot as plt
import numpy as np
with esgf_pydap.Dataset(url[0],session=session,**cred) as dataset:
    data = dataset.variables['pr'][0,:,:]
    plt.contourf(np.squeeze(data))
    plt.show()




import dask.array as da
import dask.bytes as db

import numpy as np

import netCDF4

file='/home/laliberte/CDB/2.0/sea_ice_landfast/'

dataset=netCDF4.Dataset(file)
x = da.from_array(dataset.variables['usi'],chunks=1000)
print(x.shape)



import requests
import netcdf4_soft_links.esgf_pydap as esgf_pydap
cred={'openid': 'flaliberte', 'password': 'Salut1001', 'use_certificates': False, 'authentication_url':'https://urs.earthdata.nasa.gov/'}
url = 'http://goldsmr3.gesdisc.eosdis.nasa.gov:80/opendap/MERRA_MONTHLY/MAIMCPASM.5.2.0/1979/MERRA100.prod.assim.instM_3d_asm_Cp.197901.hdf'

session=requests.Session()
import matplotlib.pyplot as plt
import numpy as np
with esgf_pydap.Dataset(url,session=session,**cred) as dataset:
    data = dataset.variables['SLP'][0,:,:]
    plt.contourf(np.squeeze(data))
    plt.show()




import os
import requests
import pydap.lib

headers = {
    'user-agent': pydap.lib.USER_AGENT,
            'connection': 'close'}
url='http://esgf-data1.ceda.ac.uk/thredds/dodsC/esg_dataroot/cmip5/output1/CCCma/CanAM4/amip/fx/atmos/fx/r0i0p0/v1/orog/orog_fx_CanAM4_amip_r0i0p0.nc'
#url='http://aims3.llnl.gov/thredds/dodsC/cmip5_css02_data/cmip5/output1/NCAR/CCSM4/amip/day/atmos/day/r4i1p1/tas/1/tas_day_CCSM4_amip_r4i1p1_19790101-20101231.nc'
session=requests.Session()
X509_PROXY=os.environ['X509_USER_PROXY']
resp=session.get(url,
                     cert=X509_PROXY,allow_redirects=True,headers=headers)
                     #cert=(X509_PROXY,X509_PROXY),allow_redirects=True,headers=headers,verify=False)
if resp.status_code==401:
    headers = {
            "User-Agent": "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/51.0.2704.84 Safari/537.36",
            "X-Requested-With": "XMLHttpRequest"}
    data={'openid_identifier':'https://ceda.ac.uk/openid/'}
    resp=session.get(url,allow_redirects=True,headers=headers,params=data)
    print(resp.headers)
    print(resp.status_code)
    #print(resp.text)




import requests
import mechanize
import cookielib
import urllib

def get_cookies(openid,password):
    issuer_node='/'.join(openid.split('/')[:3]).replace('http:','https:')

    br = mechanize.Browser()
    cj = cookielib.LWPCookieJar()
    br.set_cookiejar(cj)

    # Browser options
    br.set_handle_equiv(True)
    br.set_handle_redirect(True)
    br.set_handle_referer(True)
    br.set_handle_robots(False)

    # Follows refresh 0 but not hangs on refresh > 0
    br.set_handle_refresh(mechanize._http.HTTPRefreshProcessor(), max_time=1)

    # Want debugging messages?
    #br.set_debug_http(True)
    #br.set_debug_redirects(True)
    #br.set_debug_responses(True)

    br.addheaders = [('User-agent', 'Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.0.1) Gecko/2008071615 Fedora/3.0.1-1.fc9 Firefox/3.0.1')]

    base_url=issuer_node+'/esg-orp/j_spring_openid_security_check.htm?openid_identifier='+urllib.quote_plus(openid)

    r = br.open(base_url)
    html = r.read()

    br.select_form(nr=0)

    br.form['password']=password
    br.submit()
    return cj

if __name__=='__main__':
    openid='https://pcmdi.llnl.gov/esgf-idp/openid/flalib'
    password='Salut1001!'
    #url='http://esgf-data1.ceda.ac.uk/thredds/dodsC/esg_dataroot/cmip5/output1/CCCma/CanAM4/amip/fx/atmos/fx/r0i0p0/v1/orog/orog_fx_CanAM4_amip_r0i0p0.nc.html'
    url='http://aims3.llnl.gov/thredds/dodsC/cmip5_css02_data/cmip5/output1/NCAR/CCSM4/amip/day/atmos/day/r4i1p1/tas/1/tas_day_CCSM4_amip_r4i1p1_19790101-20101231.nc.html'
    
    session=requests.Session()
    cookies=get_cookies(openid,password)
    resp=session.get(url,allow_redirects=True)
    print(resp.text)




import requests
import pydap.lib
from bs4 import BeautifulSoup

username='flalib'
openid='https://pcmdi.llnl.gov/esgf-idp/openid/flalib'
url='http://esgf-data1.ceda.ac.uk/thredds/dodsC/esg_dataroot/cmip5/output1/CCCma/CanAM4/amip/fx/atmos/fx/r0i0p0/v1/orog/orog_fx_CanAM4_amip_r0i0p0.nc'

session=requests.Session()
resp=session.get(url,allow_redirects=True)

soup=BeautifulSoup(resp.text,'lxml')
form = soup.find('form')
auth_url=form.get('action')
if auth_url[0]=='/':
    auth_url='/'.join(url.split('/')[:4])+auth_url
resp=session.get(auth_url,allow_redirects=True)

check_url=resp.url.split('?')[0]
base_headers = {
    "Accept":'text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,*/*;q=0.8',
    "Accept-Language":'en-US,en;q=0.8',
    "Accept-Encoding":'gzip, deflate, sdch, br',
    "Upgrade-Insecure-Requests":1,
            }
base_url='https://esgf-data1.ceda.ac.uk/esg-orp/j_spring_openid_security_check.htm?openid_identifier=https%3A%2F%2Fpcmdi.llnl.gov%2Fesgf-idp%2Fopenid%2Fflalib'
resp=session.get(base_url,headers=base_headers)
specific_headers=({'Host':'pcmdi.llnl.gov','Connection':'keep-alive'})
next_to_last_resp=resp.history[-1]

headers = {
    "Accept":'text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,*/*;q=0.8',
    "Accept-Language":'en-US,en;q=0.8',
    "Accept-Encoding":'gzip, deflate, sdch, br',
    'Cache-Control':'max-age=0',
    'Host':'pcmdi.llnl.gov',
    'Connection':'keep-alive',
    'Cookie':next_to_last_resp.headers['Set-Cookie'].split(';')[0],
    "Upgrade-Insecure-Requests":1,
            }
resp=session.get(resp.url,headers=headers,allow_redirects=True,cookies=next_to_last_resp.cookies)

soup=BeautifulSoup(resp.text,'lxml')
form = soup.find('form')
auth_url=form.get('action')
if auth_url[0]=='/':
    auth_url='/'.join(resp.url.split('/')[:4])+auth_url
headers = {
    "Accept":'text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,*/*;q=0.8',
    "Accept-Language":'en-US,en;q=0.8',
    "Accept-Encoding":'gzip, deflate, br',
    'Cache-Control':'max-age=0',
    "Content-Type":"application/x-www-form-urlencoded",
    'Host':'pcmdi.llnl.gov',
    'Origin':'https://pcmdi.llnl.gov',
    'Referer':auth_url,
    'Connection':'keep-alive',
    'Content-Length':'21',
    "Upgrade-Insecure-Requests":1,
            }

payload={'password':'Salut1001!'}
print(headers)
resp=session.post(auth_url,data=payload,headers=headers,allow_redirects=True,cookies=next_to_last_resp.cookies)
print(resp.headers)
print(resp)
quit()

final_url='https://esgf-data1.ceda.ac.uk/esg-orp/home.htm'
headers={
'Accept':'text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,*/*;q=0.8',
'Accept-Encoding':'gzip, deflate, sdch, br',
'Accept-Language':'en-US,en;q=0.8',
'Cache-Control':'max-age=0',
'Connection':'keep-alive',
'Cookie':next_to_last_resp.headers['Set-Cookie'].split(';')[0],
'Host':'esgf-data1.ceda.ac.uk',
'Referer':auth_url,
'Upgrade-Insecure-Requests':1,
'User-Agent':'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_11_6) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/51.0.2704.103 Safari/537.36'
}
resp=session.get(final_url,headers=headers,allow_redirects=True,cookies=next_to_last_resp.cookies)
quit()


headers = {
    'user-agent': pydap.lib.USER_AGENT}
url='http://esgf-data1.ceda.ac.uk/thredds/dodsC/esg_dataroot/cmip5/output1/CCCma/CanAM4/amip/fx/atmos/fx/r0i0p0/v1/orog/orog_fx_CanAM4_amip_r0i0p0.nc'
resp=session.get(url,allow_redirects=True,headers=headers)
quit()

headers = {
    "Accept":'text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,*/*;q=0.8',
    "Accept-Language":'en-US,en;q=0.8',
    "Accept-Encoding":'gzip, deflate, sdch, br',
    'Host':'esgf-data1.ceda.ac.uk',
    "Upgrade-Insecure-Requests":1,
    "User-Agent":"Mozilla/5.0 (Macintosh; Intel Mac OS X 10_11_6) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/51.0.2704.103 Safari/537.36"
            }
resp=session.get(base_url,headers=headers)
#print(resp.text)
quit()

url='http://esgf-data1.ceda.ac.uk/thredds/dodsC/esg_dataroot/cmip5/output1/CCCma/CanAM4/amip/fx/atmos/fx/r0i0p0/v1/orog/orog_fx_CanAM4_amip_r0i0p0.nc'
resp=session.get(url,allow_redirects=True,headers=headers)
print(resp.status_code)
quit()




import mechanize

#base_url = 'http://goldsmr3.gesdisc.eosdis.nasa.gov:80/opendap/MERRA_MONTHLY/MAIMCPASM.5.2.0/1979/MERRA100.prod.assim.instM_3d_asm_Cp.197901.hdf'
base_url = 'https://urs.earthdata.nasa.gov/'

br = mechanize.Browser()
#br.set_cookiejar(cj)

# Browser options
br.set_handle_equiv(True)
br.set_handle_redirect(True)
br.set_handle_referer(True)
br.set_handle_robots(False)

# Follows refresh 0 but not hangs on refresh > 0
br.set_handle_refresh(mechanize._http.HTTPRefreshProcessor(), max_time=1)

# Want debugging messages?
br.set_debug_http(True)
br.set_debug_redirects(True)
br.set_debug_responses(True)

br.addheaders = [('User-agent', 'Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.0.1) Gecko/2008071615 Fedora/3.0.1-1.fc9 Firefox/3.0.1')]

r = br.open(base_url)
html = r.read()

br.select_form(nr=0)

br.form['username']='flaliberte'
br.form['password']='Salut1001'

r=br.submit()

html=r.read()
print(html)
br.close()



import h5netcdf.legacyapi as netCDF4_h5
#import netCDF4

#file_name='orog_fx_MRI-CGCM3_amip_r0i0p0.nc'
file_name='test.nc'
#dataset=netCDF4.Dataset(file_name,'r')
dataset=netCDF4_h5.Dataset(file_name,'r')
groups =dataset.groups.keys()
print(groups)

dataset.close()

quit()



#External:
import sys

#Internal:
import certificates
import manage_soft_links_class
import manage_soft_links_parsers

def main(string_call=None):
    import argparse 
    import textwrap

    #Option parser
    version_num='0.7.3'
    description=textwrap.dedent('''\
    This script aggregates soft links to OPENDAP or local files.\
    ''')
    epilog='Version {0}: Frederic Laliberte (03/2016),\n\
Previous versions: Frederic Laliberte, Paul Kushner (2011-2016).\n\
\n\
If using this code to retrieve and process data from the ESGF please cite:\n\n\
Efficient, robust and timely analysis of Earth System Models: a database-query approach (2016):\n\
F. Laliberte, Juckes, M., Denvil, S., Kushner, P. J., TBD, Submitted.'.format(version_num)
    parser = argparse.ArgumentParser(formatter_class=argparse.RawDescriptionHelpFormatter,
                            description=description,
                            version='%(prog)s '+version_num,
                            epilog=epilog)


    #Generate subparsers
    manage_soft_links_parsers.generate_subparsers(parser,epilog,None)

    if string_call != None:
        options=parser.parse_args(string_call)
    else:
        options=parser.parse_args()

    options=certificates.prompt_for_username_and_password(options)

    if options.command!='certificates':
        getattr(manage_soft_links_class,options.command)(options)
        
if __name__ == "__main__":
    main()



import requests
import mechanize
import cookielib
import urllib
import ssl
import datetime
import os

from netcdf4_soft_links import esgf_pydap
from netcdf4_soft_links import esgf_http
from netcdf4_soft_links import esgf_get_cookies

if __name__=='__main__':
    #openid='https://pcmdi.llnl.gov/esgf-idp/openid/flalib'
    #password='Salut1001!'
    openid='https://ceda.ac.uk/openid/Frederic.Laliberte'
    username='flaliberte'
    password='Salut1001'
    url_list=[
    'http://esgf-data1.ceda.ac.uk/thredds/dodsC/esg_dataroot/cmip5/output1/CCCma/CanAM4/amip/fx/atmos/fx/r0i0p0/v1/orog/orog_fx_CanAM4_amip_r0i0p0.nc',
    'http://esgf-data1.diasjp.net/thredds/dodsC/esg_dataroot/cmip5/output1/MRI/MRI-CGCM3/amip/fx/atmos/fx/r0i0p0/v20110831/orog/orog_fx_MRI-CGCM3_amip_r0i0p0.nc',
    'http://esgf-data1.ceda.ac.uk/thredds/dodsC/esg_dataroot/cmip5/output1/NCAR/CCSM4/amip/fx/atmos/fx/r0i0p0/v20130312/orog/orog_fx_CCSM4_amip_r0i0p0.nc',
    'http://esgf2.dkrz.de/thredds/dodsC/cmip5/output1/NCAR/CCSM4/amip/fx/atmos/fx/r0i0p0/v20130312/orog/orog_fx_CCSM4_amip_r0i0p0.nc',
    'http://aims3.llnl.gov/thredds/dodsC/cmip5_css02_data/cmip5/output1/NCAR/CCSM4/amip/day/atmos/day/r1i1p1/tas/1/tas_day_CCSM4_amip_r1i1p1_19790101-20101231.nc',
    ]
    
    session=requests.Session()
    for url in url_list:
        print(url)
        time_start=datetime.datetime.now()
        with esgf_pydap.Dataset(url,session=session,openid=openid,password=password,username=username) as dataset:
            print(dataset)
            print(len(session.cookies))
            print(datetime.datetime.now()-time_start)

    session=requests.Session()
    for url in url_list:
        print(url)
        time_start=datetime.datetime.now()
        with esgf_http.Dataset(url.replace('dodsC','fileServer'),session=session,openid=openid,password=password,username=username) as dataset:
            size_string=dataset.wget(os.path.basename(url))
            print(size_string)
            print(len(session.cookies))
            print(datetime.datetime.now()-time_start)






import requests
import pydap.lib

url='https://ceda.ac.uk/OpenID/Provider/login'
session=requests.Session()
#resp=session.get(url,allow_redirects=True)
#print(resp.status_code)
payload={'username':'flaliberte','password':'Salut1001',
         'submit':'Login',
         'success_to':'https://ceda.ac.uk/OpenID/Provider/home',
         'fail_to':'https://ceda.ac.uk/OpenID/Provider/login'
         }
resp=session.post(url,data=payload,allow_redirects=True)
print(resp.text)
print(resp.status_code)



headers = {
    'user-agent': pydap.lib.USER_AGENT,
        'connection': 'close'}
url='http://esgf-data1.ceda.ac.uk/thredds/dodsC/esg_dataroot/cmip5/output1/CCCma/CanAM4/amip/fx/atmos/fx/r0i0p0/v1/orog/orog_fx_CanAM4_amip_r0i0p0.nc'
resp=session.get(url,allow_redirects=True,headers=headers)
print(resp.status_code)



import requests
import pydap.lib
from bs4 import BeautifulSoup

username='flalib'
openid='https://pcmdi.llnl.gov/esgf-idp/openid/'
url='https://pcmdi.llnl.gov/openid/login/'
#url'https://esgf-index1.ceda.ac.uk/openid/login/'
session=requests.Session()
#resp=session.get(url,allow_redirects=True)
resp=session.get(url)
csrftoken = session.cookies['csrftoken']

payload={'openid_identifier':openid+'flalib', 'csrfmiddlewaretoken':csrftoken}
#payload={}
#for input in inputs:
#    name=input.get('name')
#    value=input.get('value')
#    if name=='openid_identifier':
#        payload.update({name:openid})
#    elif input.get('type') != 'submit':
#        payload.update({name:value})
print(payload)
resp=session.post(url,data=payload,allow_redirects=True,headers=dict(Referer=url))

soup=BeautifulSoup(resp.text,'lxml')
form = soup.find('form')
auth_url=form.get('action')
inputs = form.find_all('input')
payload={}
for input in inputs:
    name=input.get('name')
    value=input.get('value')
    payload.update({name:value})
print(payload)

resp=session.post(auth_url,data=payload,allow_redirects=True,headers=dict(Referer=url))
soup=BeautifulSoup(resp.text,'lxml')
form=soup.find('form')
print(auth_url)
submit_url=form.get('action')
if submit_url[0]=='/':
    submit_url='/'.join(auth_url.split('/')[:4])+submit_url
print(submit_url)

inputs = form.find_all('input')
payload={'password':'Salut1001!'}

#resp=session.post(submit_url,data=payload,allow_redirects=True,headers=dict(Referer=url))
resp=session.post(submit_url,data=payload,allow_redirects=True,headers=dict(Referer=url))
print(resp.text)
quit()
headers = {
    'user-agent': pydap.lib.USER_AGENT,
        'connection': 'close'}
url='http://esgf-data1.ceda.ac.uk/thredds/dodsC/esg_dataroot/cmip5/output1/CCCma/CanAM4/amip/fx/atmos/fx/r0i0p0/v1/orog/orog_fx_CanAM4_amip_r0i0p0.nc'
resp=session.get(url,allow_redirects=True,headers=headers)
print(resp.status_code)
